@@ -73,12 +73,6 @@ module Filesystem
 	, rename
 	) where
 
-#ifndef CABAL_OS_WINDOWS
-#if MIN_VERSION_base(4,2,0)
-#define SYSTEMFILEIO_LOCAL_OPEN_FILE
-#endif
-#endif
-
 import           Prelude hiding (FilePath, readFile, writeFile, appendFile)
 
 import qualified Control.Exception as Exc
@@ -118,6 +112,9 @@ import           Data.Time (UTCTime)
 import           Data.Time.Clock.POSIX (posixSecondsToUTCTime)
 import qualified System.Posix as Posix
 import qualified System.Posix.Error as Posix
+#if MIN_VERSION_unix(2,5,1)
+import qualified System.Posix.Files.ByteString
+#endif
 
 #endif
 
@@ -158,7 +155,7 @@ isDirectory path = SD.doesDirectoryExist (encodeString path)
 #else
 isDirectory path = Exc.catch
 	(do
-		stat <- posixStat "isFile" path
+		stat <- posixStat "isDirectory" path
 		return (Posix.isDirectory stat))
 	((\_ -> return False) :: IOError -> IO Bool)
 #endif
@@ -953,18 +950,23 @@ throwErrnoPathIfRetry_ failed loc path io = do
 	_ <- throwErrnoPathIfRetry failed loc path io
 	return ()
 
+posixStat :: String -> FilePath -> IO Posix.FileStatus
+#if MIN_VERSION_unix(2,5,1)
+posixStat _ path = System.Posix.Files.ByteString.getFileStatus (R.encode R.posix path)
+#else
+posixStat loc path = withFd loc path Posix.getFdStatus
+
 withFd :: String -> FilePath -> (Posix.Fd -> IO a) -> IO a
 withFd fnName path = Exc.bracket open close where
 	open = withFilePath path $ \cpath -> do
-		fd <- throwErrnoPathIfMinus1 fnName path (c_open cpath 0)
+		fd <- throwErrnoPathIfMinus1 fnName path (c_open_nonblocking cpath 0)
 		return (Posix.Fd fd)
 	close = Posix.closeFd
 
-posixStat :: String -> FilePath -> IO Posix.FileStatus
-posixStat loc path = withFd loc path Posix.getFdStatus
+foreign import ccall unsafe "hssystemfileio_open_nonblocking"
+	c_open_nonblocking :: CString -> CInt -> IO CInt
 
-foreign import ccall unsafe "open"
-	c_open :: CString -> CInt -> IO CInt
+#endif
 
 foreign import ccall unsafe "free"
 	c_free :: Ptr a -> IO ()
